'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _toPrimitive(t, r) {
  if ("object" != typeof t || !t) return t;
  var e = t[Symbol.toPrimitive];
  if (void 0 !== e) {
    var i = e.call(t, r );
    if ("object" != typeof i) return i;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return (String )(t);
}
function _toPropertyKey(t) {
  var i = _toPrimitive(t, "string");
  return "symbol" == typeof i ? i : i + "";
}
function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor);
  }
}
function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  Object.defineProperty(Constructor, "prototype", {
    writable: false
  });
  return Constructor;
}

var CheckboxTree = /*#__PURE__*/function () {
  /**
   * Constructor for the CheckboxTree class.
   * @param {HTMLSelectElement} select - The select element to transform into a checkbox tree.
   * @param {Object} [options] - Custom options to override default settings.
   */
  function CheckboxTree(select, options) {
    this.$select = select;
    this.options = Object.assign({}, this.Defaults, options);
    this.hideSelect();
    this.buildDOM();
    this.addEventListeners();
  }

  /**
   * Hides the original select element.
   */
  var _proto = CheckboxTree.prototype;
  _proto.hideSelect = function hideSelect() {
    this.$select.classList.remove(this.options.rootClassName);
    this.$select.hidden = true;
    this.$select.disabled = true;
    this.$select.style.display = "none";
  }

  /**
   * Builds the DOM structure for the checkbox tree.
   */;
  _proto.buildDOM = function buildDOM() {
    this.$root = this._createRoot();
    this.$select.after(this.$root);
    this.$inner = this._createInner();
    this.$root.appendChild(this.$inner);
    this._fillInner();
    this._setInitialTabIndexes();
  }

  /**
   * Creates the root element of the checkbox tree.
   * @returns {HTMLDivElement} The root element.
   * @private
   */;
  _proto._createRoot = function _createRoot() {
    var root = document.createElement("div");
    root.classList.add(this.options.rootClassName);
    return root;
  }

  /**
   * Creates the inner container of the checkbox tree.
   * @returns {HTMLDivElement} The inner container.
   * @private
   */;
  _proto._createInner = function _createInner() {
    var inner = document.createElement("div");
    inner.classList.add(this.options.innerClassName);
    return inner;
  }

  /**
   * Fills the inner container with groups and options
   * based on the original select element.
   * @private
   */;
  _proto._fillInner = function _fillInner() {
    var _this = this;
    var fragment = document.createDocumentFragment();
    Array.from(this.$select.children).forEach(function (node) {
      if (node.tagName === "OPTGROUP") {
        fragment.appendChild(_this.createGroup(node));
      } else {
        fragment.appendChild(_this.createOption(node));
      }
    });
    this.$inner.appendChild(fragment);
  }

  /**
   * Creates a group element based on an OPTGROUP element.
   * @param {HTMLOptGroupElement} source - The source OPTGROUP element.
   * @returns {HTMLDivElement} The group element.
   * @private
   */;
  _proto.createGroup = function createGroup(source) {
    var _this2 = this;
    var group = document.createElement("div");
    group.classList.add(this.options.groupClassName);
    var header = this._createGroupHeader(source);
    group.appendChild(header);
    var fragment = document.createDocumentFragment();
    Array.from(source.children).forEach(function (node) {
      fragment.appendChild(_this2.createOption(node));
    });
    group.appendChild(fragment);
    this.updateGroup(group);
    return group;
  }

  /**
   * Creates the header for a group element.
   * @param {HTMLOptGroupElement} source - The source OPTGROUP element.
   * @returns {HTMLDivElement} The group header element.
   * @private
   */;
  _proto._createGroupHeader = function _createGroupHeader(source) {
    var header = document.createElement("div");
    header.classList.add(this.options.groupHeaderClassName);
    var label = document.createElement("div");
    label.classList.add(this.options.groupLabelClassName);
    header.appendChild(label);
    label.insertAdjacentHTML("beforeend", this.buildCheckboxIcon());
    var headerText = document.createElement("span");
    headerText.classList.add(this.options.groupTextClassName);
    headerText.textContent = source.label;
    label.appendChild(headerText);
    return header;
  }

  /**
   * Retrieves the checked states of options within a group.
   * @param {HTMLElement} group - The group element.
   * @returns {number[]} Array of states where 1 represents checked
   *   and 0 represents unchecked.
   * @private
   */;
  _proto._getGroupStates = function _getGroupStates(group) {
    var _this3 = this;
    var optionStates = [];
    var options = group.querySelectorAll("." + this.options.optionClassName);
    options.forEach(function (option) {
      var input = option.querySelector("." + _this3.options.optionInputClassName);
      if (input && !input.disabled) {
        optionStates.push(input.checked ? 1 : 0);
      }
    });
    return optionStates;
  }

  /**
   * Updates the visual state of a group based on its options.
   * @param {HTMLElement} group - The group element.
   * @private
   */;
  _proto.updateGroup = function updateGroup(group) {
    var header = group.querySelector("." + this.options.groupHeaderClassName);
    var icon = header.querySelector("svg");
    var groupStates = this._getGroupStates(group);
    if (groupStates.every(function (s) {
      return s === 1;
    })) {
      this.setIconState(icon, "checked");
    } else if (groupStates.some(function (s) {
      return s === 1;
    })) {
      this.setIconState(icon, "partial");
    } else {
      this.setIconState(icon, "none");
    }
  }

  /**
   * Creates an option element based on an OPTION element.
   * @param {HTMLOptionElement} source - The source OPTION element.
   * @returns {HTMLDivElement} The option element.
   * @private
   */;
  _proto.createOption = function createOption(source) {
    var option = document.createElement("div");
    option.classList.add(this.options.optionClassName);
    var label = document.createElement("label");
    label.classList.add(this.options.optionLabelClassName);
    option.appendChild(label);
    var checkbox = document.createElement("input");
    checkbox.classList.add(this.options.optionInputClassName);
    checkbox.type = "checkbox";
    checkbox.name = this.$select.name;
    checkbox.value = source.value;
    checkbox.tabIndex = -1;
    if (source.selected) {
      checkbox.toggleAttribute("checked", true);
    }
    if (source.disabled) {
      checkbox.toggleAttribute("disabled", true);
    }
    label.appendChild(checkbox);
    label.insertAdjacentHTML("beforeend", this.buildCheckboxIcon());
    var labelText = document.createElement("span");
    labelText.classList.add(this.options.optionTextClassName);
    labelText.textContent = source.textContent;
    label.appendChild(labelText);
    this.updateOption(option);
    return option;
  }

  /**
   * Updates the visual state of an option element.
   * @param {HTMLElement} option - The option element.
   * @private
   */;
  _proto.updateOption = function updateOption(option) {
    var checkbox = option.querySelector("." + this.options.optionInputClassName);
    var icon = option.querySelector("svg");
    icon.classList.toggle(this.options.checkboxCheckedClassName, checkbox.checked);
    icon.classList.toggle(this.options.checkboxDisabledClassName, checkbox.disabled);
  }

  /**
   * Builds the SVG icon for checkboxes.
   * @returns {string} The SVG markup.
   */;
  _proto.buildCheckboxIcon = function buildCheckboxIcon() {
    return "\n            <svg class=\"pct-checkbox\" width=\"22\" height=\"22\" fill=\"none\">\n              <rect class=\"pct-checkbox__box\"></rect>\n              <path class=\"pct-checkbox__marker\" d=\"M4,13 8,17 18,6\"></path>\n              <path class=\"pct-checkbox__line\" d=\"M5,11 17,11\"></path>\n            </svg>\n        ";
  }

  /**
   * Sets the visual state of an icon.
   * @param {SVGElement} icon - The icon element.
   * @param {"none"|"checked"|"partial"} state - The state to set.
   * @private
   */;
  _proto.setIconState = function setIconState(icon, state) {
    icon.classList.toggle(this.options.checkboxCheckedClassName, state === "checked");
    icon.classList.toggle(this.options.checkboxIndeterminateClassName, state === "partial");
  }

  /**
   * Sets the tab index for focusable elements.
   * @private
   */;
  _proto._setInitialTabIndexes = function _setInitialTabIndexes() {
    var focusableElements = this._getFocusableElements();
    if (!focusableElements.length) {
      return;
    }
    focusableElements[0].tabIndex = 0;
    focusableElements.slice(1).forEach(function (element) {
      element.tabIndex = -1;
    });
  }

  /**
   * Retrieves the focusable elements within the checkbox tree.
   * @returns {HTMLElement[]} Array of focusable elements.
   * @private
   */;
  _proto._getFocusableElements = function _getFocusableElements() {
    var _this4 = this;
    var labels = this.$inner.querySelectorAll("." + this.options.optionLabelClassName + ", ." + this.options.groupLabelClassName);
    return Array.from(labels).filter(function (label) {
      var input = label.querySelector("." + _this4.options.optionInputClassName);
      return !input || !input.disabled;
    });
  }

  /**
   * Adds event listeners to the checkbox tree.
   * @private
   */;
  _proto.addEventListeners = function addEventListeners() {
    var _this5 = this;
    // Add input event listener for checkbox state changes
    this.$root.addEventListener("input", function (event) {
      var optionLabel = event.target.closest("." + _this5.options.optionLabelClassName);
      if (optionLabel) {
        var option = optionLabel.closest("." + _this5.options.optionClassName);
        if (option) {
          _this5.onClickOption(option);
        }
      }
    });

    // Add click event listener for group clicks
    this.$root.addEventListener("click", function (event) {
      var groupLabel = event.target.closest("." + _this5.options.groupLabelClassName);
      if (groupLabel) {
        var group = groupLabel.closest("." + _this5.options.groupClassName);
        if (group) {
          _this5.onClickGroup(group);
        }
      }
    });

    // Add keypress event listener for Enter and Space keypresses
    this.$root.addEventListener("keypress", function (event) {
      if ([13, 32].includes(event.keyCode)) {
        var option = event.target.closest("." + _this5.options.optionClassName);
        if (option) {
          event.preventDefault();
          var input = option.querySelector("." + _this5.options.optionInputClassName);
          if (input && !input.disabled) {
            input.checked = !input.checked;
          }
          _this5.onClickOption(option);
          return;
        }
        var group = event.target.closest("." + _this5.options.groupClassName);
        if (group) {
          event.preventDefault();
          _this5.onClickGroup(group);
        }
      }
    });

    // Add pointerup event listener for tabindex handling
    this.$root.addEventListener("pointerup", function (event) {
      var label = event.target.closest("." + _this5.options.optionLabelClassName + ", ." + _this5.options.groupLabelClassName);
      if (label) {
        var input = label.querySelector("." + _this5.options.optionInputClassName);
        if (input && input.disabled) {
          return;
        }
        var focusableElements = _this5._getFocusableElements();
        focusableElements.forEach(function (element) {
          element.tabIndex = -1;
        });
        label.tabIndex = 0;
        setTimeout(function () {
          label.focus();
        });
      }
    });

    // Add keydown event listener for arrow key navigation
    this.$root.addEventListener("keydown", function (event) {
      if ([37, 38, 39, 40].includes(event.keyCode)) {
        event.preventDefault();
        var focusableElements = _this5._getFocusableElements();
        var currentElementIndex = focusableElements.indexOf(event.target);
        if ([37, 38].includes(event.keyCode)) {
          var previousElement = focusableElements[currentElementIndex - 1];
          if (previousElement) {
            focusableElements[currentElementIndex].tabIndex = -1;
            previousElement.tabIndex = 0;
            previousElement.focus();
          }
        } else {
          var nextElement = focusableElements[currentElementIndex + 1];
          if (nextElement) {
            focusableElements[currentElementIndex].tabIndex = -1;
            nextElement.tabIndex = 0;
            nextElement.focus();
          }
        }
      }
    });
  }

  /**
   * Handles click events on option elements.
   * @param {HTMLElement} option - The clicked option element.
   */;
  _proto.onClickOption = function onClickOption(option) {
    this.updateOption(option);
    var group = option.closest("." + this.options.groupClassName);
    if (group) {
      this.updateGroup(group);
    }
  }

  /**
   * Handles click events on group elements.
   * @param {HTMLElement} group - The clicked group element.
   */;
  _proto.onClickGroup = function onClickGroup(group) {
    var _this6 = this;
    var header = group.querySelector("." + this.options.groupHeaderClassName);
    var groupIcon = header.querySelector("svg");
    var options = group.querySelectorAll("." + this.options.optionClassName);
    var groupStates = this._getGroupStates(group);
    if (groupStates.every(function (s) {
      return s === 1;
    })) {
      options.forEach(function (option) {
        _this6.setOptionState(option, "none");
      });
      this.setIconState(groupIcon, "none");
    } else {
      options.forEach(function (option) {
        _this6.setOptionState(option, "checked");
      });
      this.setIconState(groupIcon, "checked");
    }
  }

  /**
   * Sets the state of an option element.
   * @param {HTMLElement} option - The option element.
   * @param {"none"|"checked"|"partial"} state - The state to set.
   */;
  _proto.setOptionState = function setOptionState(option, state) {
    var input = option.querySelector("." + this.options.optionInputClassName);
    if (input && !input.disabled) {
      input.checked = state === "checked";
      this.updateOption(option);
    }
  };
  return _createClass(CheckboxTree, [{
    key: "Defaults",
    get:
    /**
     * Default configuration for the CheckboxTree component.
     * @returns {Object} The default CSS class names for various elements.
     */
    function get() {
      return {
        rootClassName: "pct-tree",
        innerClassName: "pct-tree__inner",
        groupClassName: "pct-group",
        groupHeaderClassName: "pct-group__header",
        groupLabelClassName: "pct-group__label",
        groupTextClassName: "pct-group__text",
        optionClassName: "pct-option",
        optionLabelClassName: "pct-option__label",
        optionInputClassName: "pct-option__input",
        optionTextClassName: "pct-option__text",
        checkboxCheckedClassName: "pct-checkbox--checked",
        checkboxIndeterminateClassName: "pct-checkbox--partial",
        checkboxDisabledClassName: "pct-checkbox--disabled"
      };
    }
  }]);
}();
document.addEventListener("DOMContentLoaded", function () {
  var selects = document.querySelectorAll("select.pct-tree");
  selects.forEach(function (select) {
    return new CheckboxTree(select);
  });
});

exports.default = CheckboxTree;
module.exports = Object.assign(exports.default, exports);
//# sourceMappingURL=cjs.js.map
